#!/usr/bin/perl
use strict;
use warnings;

# Global variables ==========
my $FIRST_AUTHOR_SCORE = 0.8;
my $SORT = 1;
my %score_of;
my %count_of;
# ===========================


# Obtain list
my $dir = $ARGV[0] or die "Input directory.\n";
$dir =~ s#/$##;

# Sorting order 1: Count, 2: Score
$SORT = $ARGV[1] if $ARGV[1];


my $cmd = "find $dir -name \"*.html\"";
my $cmd_result = `$cmd`;

foreach my $file (split /\n/, $cmd_result) {
  next unless ($file =~ /\.html$/);
#  print $file, "\n";
#  next;

  open my $IN, '<', "$file" or die "Cannot open $file\n";
  foreach my $line (<$IN>) {
    if ($line =~ /<li>/) {
      my @author_list;
      while ($line =~ m#<a[^>]+>([^<]+)</a>#g) {
	#      print $1, " AND ";
	push @author_list, $1;
      }
      divide_score(\%score_of, \%count_of, \@author_list);
    }
  }
  close($IN);
}


# Output
my @sorted_authors;
if ($SORT == 1) {
  @sorted_authors = sort {
    $count_of{$b} <=> $count_of{$a}
  } keys %count_of;
} else {
  @sorted_authors = sort {
    $score_of{$b} <=> $score_of{$a}
  } keys %score_of;
}

foreach my $author (@sorted_authors) {
  print "$author\t$count_of{$author}\t$score_of{$author}\n";
}



# Receive author list and return score of them
sub divide_score {
  my ($score_of_ref, $count_of_ref, $author_list_ref) = @_;

  # If there is only one author, return 1.0
  my $author_num = scalar @$author_list_ref;

  return 0 if ($author_num < 1);

  if ($author_num == 1) {
    $score_of_ref->{ $author_list_ref->[0] } += 1.0;
    $count_of_ref->{ $author_list_ref->[0] } += 1;
    return 1;
  }

  # Add first author's score
  $score_of_ref->{ $author_list_ref->[0] } += $FIRST_AUTHOR_SCORE;
  $count_of_ref->{ $author_list_ref->[0] } += 1;

  # Calculate other authors' score
  my $rest_score = (1 - $FIRST_AUTHOR_SCORE) / ($author_num - 1);
  for (my $i = 1; $i < $author_num; $i++) {
    $score_of_ref->{ $author_list_ref->[ $i ] } += $rest_score;
    $count_of_ref->{ $author_list_ref->[ $i ] } += 1;
  }

  return 1;
}
